@@ -100,7 +100,8 @@ class WorkerCore(ABC):
         self.monitor.record_worker_info()
 
     def _add_arguments(self, parser):
-        parser.add_argument('--debug', action='store_true', default=os.environ.get('DEBUG', 'FALSE') == 'TRUE',
+        parser.add_argument('--debug', action='store_true',
+                            default=os.environ.get('DEBUG', 'FALSE') == 'TRUE',
                             help='debug, more verbose logging')
         if self.flag != WorkerConfig.NO_INPUT:
             self.sourceClass.add_arguments(parser)
@@ -210,6 +211,7 @@ class Splitter(WorkerCore):
             destination = self.destinations[topic]
 
             if msg.is_valid():
+                self.logger.info("Writing message %s to topic <%s>", str(msg), topic)
                 destination.write(msg)
             else:
                 self.logger.error('Produced message is invalid, skipping')
@@ -311,7 +313,7 @@ class Processor(WorkerCore):
             else:
                 msg.update_version(self.name, self.version)
         else:
-            self.logger.warn('Message has been processed by higher version processor, no processed')
+            self.logger.warning('Message has been processed by higher version processor, no processed')
 
         # skip validation and output for some cases
         if msg.terminated:
@@ -324,12 +326,12 @@ class Processor(WorkerCore):
             else:
                 failedOnError = True
                 self.logger.error('result message is invalid, skipping')
-                self.logger.warn(msg.log_info())
-                self.logger.warn(msg.log_content())
+                self.logger.warning(msg.log_info())
+                self.logger.warning(msg.log_content())
 
         # retry if necessary
         if failedOnError and self.retryEnabled:
-            self.logger.warn('message is sent to retry topic %s', self.retryDestination.config.out_topic)
+            self.logger.warning('message is sent to retry topic %s', self.retryDestination.config.out_topic)
             self.retryDestination.write(msg)
             self.monitor.record_write(self.retryDestination.topic)
 
